class GalleriesController < ApplicationController

  def index
    @galleries = Gallery.find(:all)
    @last_images = Image.find(:all,:limit => 4, :order => "created_at DESC")
  end

  def show
    @gallery = Gallery.find_by_permalink params[:permalink]
    raise_404 if @gallery.blank?
    @images = @gallery.images
  end

  def add
    dir = params[:id]
    path = File.join(IMAGES_PATH,dir)
    @already_exist = ! Gallery.find_by_dirname(dir).blank?
    if(
        !@already_exist &&
        File.exist?(path) &&
        File.directory?(path)
      )
      g = Gallery.new(:dirname => dir, :name => dir)
      g.save
      redirect_to edit_gallery_path(dir)
    else
      flash[:error] = "This is not directory or gallery already exist!"
      redirect_to("/galleries/")
    end
  end

  def import_all
    @gallery = Gallery.find_by_dirname params[:id]
    @images = get_images(@gallery);
    @images[:new].each do |i|
      img = Image.new(:name => i, :filename => i, :gallery => @gallery)
      img.save
      @gallery.images << img
    end
    redirect_to edit_gallery_path(@gallery.dirname)
  end

  def edit
    @gallery = Gallery.find_by_dirname params[:id]
    @images = get_images(@gallery);
    @new_images = @images[:new]; @old_images = @images[:old];
  end

  def update
    @gallery = Gallery.find_by_dirname params[:gallery][:dirname]
    @gallery.update_attributes(params[:gallery])
    @gallery.save
    flash[:message] = "Gallery updated!";
    redirect_to edit_gallery_path(@gallery.dirname)
  end

  private
  def get_images(gallery)
    images = gallery.images
    new_images = []; old_images = [];
    Dir[File.join(IMAGES_PATH,gallery.dirname,"*.jpg")].each do |filename|
      next if File.directory?(filename)
      basename = File.basename(filename)
      (images.select { |i| i.filename == basename }.blank? ? new_images : old_images) << basename
    end
    return { :new => new_images, :old => old_images }
  end

end
