#require 'RMagick'

class Scanner

  attr_accessor :path, :tn_path

  def initialize(source, images, thumbs)
    @source_path, @images_path, @thumbs_path = source, images, thumbs
  end

  def scan
    get_all_existed
    scan_new_galleries
    get_all_existed
    scan_new_images
  end

  private
  def resize(source,target,size)

    FileUtils.mkdir_p File.dirname(target)

    ########## While RMagic doesn't work
#    FileUtils.copy_file(source, target)
#    puts "Generated[dummy]: #{source} -> #{target} (#{size[:width]}x#{size[:height]})"
#    return
    ##########


    wm = Magick::Image.read(RAILS_ROOT + size[:overlay]).first unless size[:overlay].blank?

    i = Magick::ImageList.new(source)
    i.resize_to_fit!(size[:width],size[:height])
    unless size[:overlay].blank?
      i = i.composite(wm, Magick::SouthEastGravity, Magick::OverCompositeOp)
    end
    i.write target

    puts "Generated: #{source} -> #{target} (#{size[:width]}x#{size[:height]})"
  end

  #Scan galleries directory for new images
  def scan_new_images
    @galleries.each do |gallery|
      gallery_images = @images.select{|i| i.gallery_id == gallery.id}
      Dir[File.join(@source_path,gallery.dirname,"*.jpg")].each do |image_file|
        filename = File.basename(image_file)
        unless gallery_images.select{|i| i.filename == filename}.blank?
          puts "Image '#{image_file}' in gallery '#{gallery.dirname}' already exits"
          next
        end
        img = Image.new(:name => filename, :filename => filename)
        img.save
        gallery.images << img
        gallery.save

        resize(File.join(@source_path,gallery.dirname,filename),
          File.join(@images_path,gallery.dirname,filename),
          {:width => 600, :height => 600}
        )
        resize(File.join(@source_path,gallery.dirname,filename),
          File.join(@thumbs_path,gallery.dirname,filename),
          {:width => 180, :height => 180}
        )

        puts "New image '#{image_file}' in gallery '#{gallery.dirname}'"
      end
    end
  end

  #Get all galleries and images from DB
  def get_all_existed
    @galleries = Gallery.find(:all)
    @images = Image.find(:all)
  end

  #Find directories with new galleries and add them to DB
  def scan_new_galleries
    Dir[File.join(@source_path,"*")].each do |dir|
      next unless File.directory?(dir)
      dirname = File.basename(dir)

      #skip if already exist
      unless @galleries.select { |g| g.dirname == dirname }.blank?
        puts "Directory '#{dirname}' is already exist - skip it."
        next
      end

      #add of new
      g = Gallery.new(:dirname => dirname, :name => dirname.humanize.capitalize, :permalink => dirname.permalink)
      g.save
      puts "Directory '#{dirname}' is new - add it."
    end
  end
end
